/*
 * File: SumOfPrimeFactors.c
 *
 * Author: M. I. Karavelas
 * Class:  EM101 / Fall Semester 2008
 */

#include "SumOfPrimeFactors.h"
#include "IsPrime.h"

/*
 * Function: SumOfPrimeFactors
 * Usage: sum = SumOfPrimeFactors(n);
 * ----------------------------------
 * Returns the sum of the prime factors of n.
 *
 * The algorithm works as follows:
 * We first divide n by the proper power of 2 (i.e., we check 2 as a
 * potential prime factor). Then the remaining part of n, which is now
 * odd, is checked against all possible divisors from 3 to the maximum
 * prime factor of n. For each divisor found (which has to be prime)
 * we repeatedly divide n by it, so as eliminate this factor.
 */

unsigned int SumOfPrimeFactors(unsigned int n)
{
  unsigned int sum = 0, divisor = 3;
  bool is_prime;

  /* first consider 2 as a prime factor */
  while ( n % 2 == 0 ) {
    sum += 2;
    n /= 2;
  }

  /* check if n was a power of 2; in that case return the already
     computed sum */
  if ( n == 1 ) { return sum; }

  /* initialize is_prime for the while-loop that follows */
  is_prime = IsPrime(n);

  while ( n > 1 ) {
    /* if the remaining portion of the number is prime, we need not do
       anything else */
    if ( is_prime ) { return sum + n; }

    /* otherwise check for next available divisor */
    if ( n % divisor == 0 ) {
      /* note that this portion of the code is executed only if
	 "divisor" is a prime number;
	 to see this consider that "divisor" is not prime; then we must
	 have checked against all its divisors earlier, which implies
	 that we have divided n by all divisors of "divisor", and thus
	 it would have been possible for "divisor" to divide n.
      */
      sum += divisor;
      n /= divisor;

      while ( n % divisor == 0 ) {
	sum += divisor;
	n /= divisor;
      }

      is_prime = IsPrime(n);
    }

    /* clearly we need to check only odd divisors (all primes greater
       than 2 are odd) */
    divisor += 2;
  }

  return sum;
}
